function rfrebme_data = rfrebme(old_rfrebme_data, model_ext_row, measure_ext)
%RFREBME Recursive full-rank ellipsoidal blind minimax estimator.
%   RFREBME_DATA = RFREBME(OLD_RFREBME_DATA, H_EXT, Y_EXT) is a struct
%   containing the data required to recursively calculate the full-rank
%   ellipsoidal blind minimax estimation. RFREBME_DATA.estimation is the
%   current estimate of the parameter vector. It is updated from 
%   OLD_RFREBME_DATA using H_EXT and Y_EXT, the extensions to the model 
%   matrix and measurements vector, respectively.
%
%   In other words, updated is the full-rank ellipsoidal blind minimax
%   estimate of the vector X in the problem Y_n = H_n * X + W_n , where Y_n
%   is the measurements vector, H_n is the model matrix and W_n is white
%   Gaussian noise. Y_EXT is the measurement added to the bottom of Y_{n-1}
%   to create Y_n, while H_EXT is the transpose of the row added to the
%   bottom of H_{n-1} to create H_n.
%
%   See also RSBME_INIT, RSBME, RSBME_INIT.

%   Asaf Elron and Guy Leibovitz, 2008.
%   Based on Leibovitz, Elron, Ben-Haim & Eldar, Recursive Blind Minimax
%   Estimation: Improving MSE over Recursive Least Squares", IEEEI 2008.

%% get stuff

% save old P
old_P = old_rfrebme_data.rls_data.P; %#ok<NASGU>

% update RLS estimation and intermediate data
rls_data = rls(old_rfrebme_data.rls_data, model_ext_row, measure_ext);

% get local copies, for simpler code
P = rls_data.P; % P = noise_variance^-1 * Q^-1
phi = rls_data.phi;
phi_again = rls_data.phi_again;
gamma = rls_data.gamma;
lambda = rls_data.forgetting_factor;
noise_variance = rls_data.noise_variance;

% save old P^2
old_P_to_power_2 = old_rfrebme_data.P_to_power_2;

%% update

% update P^2 - as given in paper
% sum_with_own_transpose = @(X) X+transpose(X);
% P_to_power_2 = (old_P_to_power_2 ...
%                 - sum_with_own_transpose(old_P*phi*phi')*gamma ...
%                 + phi'*phi*phi*phi'*gamma^2)/lambda^2;

% update P^2 - numerically stabilized
P_to_power_2 = (old_P_to_power_2 ...
                - gamma*phi*model_ext_row'*old_P_to_power_2 ...
                - gamma*old_P_to_power_2*model_ext_row*phi_again ...
                + gamma^2*phi*model_ext_row'*old_P_to_power_2*model_ext_row*phi_again) / lambda^2;

% calculate actual estimation x_hat_rfrebm
r1 = trace(P_to_power_2) * noise_variance^2;
r2 = sum(sum(P_to_power_2'.*P))*noise_variance^3;
x_hat_ls_squared_Ptob_norm = rls_data.estimation'*P_to_power_2*rls_data.estimation * noise_variance^2;

alpha = r1 / (x_hat_ls_squared_Ptob_norm + r2);

x_hat_rebm = (eye(size(P,1)) - alpha*P * noise_variance) * rls_data.estimation;

% save updated tracking struct
rfrebme_data = struct('rls_data', rls_data, ...
                      'P_to_power_2', P_to_power_2, ...
                      'estimation', x_hat_rebm);

end
