function rsbme_data = rsbme_init(parameter_count, delta, lambda, noise_variance)
%RSBME_INIT Initialize recursive spherical blind minimax estimator.
%   RSBME_DATA = RSBME_INIT(PARAMETER_COUNT, DELTA, LAMBDA, NOISE_VAR) is a
%   struct containing preperatory data for recursively calculating the 
%   spherical blind minimax estimatator, to be used with RSBME().
%   PARAMETER_COUNT is the number of paramters to estimate. DELTA is either
%   the regularization initialization parameter [1] or initial estimation
%   data created by INITIAL_ESTIMATION(). LAMBDA is the forgetting factor.
%   NOISE_VAR is the variance of the additive noise.
%
%   [1] Haykin, Adaptive Filter Theory, 4th ed., pp. 437.
%
%   See also RSBME, RFREBME_INIT, RFREBME.

%   Asaf Elron and Guy Leibovitz, 2008.
%   Based on Leibovitz, Elron, Ben-Haim & Eldar, Recursive Blind Minimax
%   Estimation: Improving MSE over Recursive Least Squares", IEEEI 2008.

%% 

% initialize RLS
rls_data = rls_init(parameter_count, delta, lambda, noise_variance);
x_hat_ls = rls_data.estimation; % get local copy

epsilon0 = trace(rls_data.P) * rls_data.noise_variance; % MSE of LS (3.5)

L_square = norm_squared(x_hat_ls); % calculate x_hat_ls's squared norm

x_hat_sbm = L_square / (L_square + epsilon0) * x_hat_ls; % calculate SBM estimation (3.7)

% generate tracking struct
rsbme_data = struct('rls_data', rls_data, ...
                    'estimation', x_hat_sbm);

end
