function [sbme_data, effective_dimension] = sbme_prep(H_or_LS, Cw_or_Qinv, option)
%SBME_PREP Prepare spherical blind minimax estimator.
%   SBME = SBME_PREP(H, Cw) is a struct containing preperations for
%   the calculation of the spherical blind minimax estimation, to be used
%   with EBME().
%   The measurement transform H is assumed to be of full rank and the noise
%   covariance Cw is assumed to be positive definite.
%
%   SBME = SBME_PREP(LS, Qinv, 1) may be used if LEASTSQ_PREP() has
%   been used prior to SBME_PREP. Otherwise, it is identical in behavior to
%   SBME_PREP(H, Cw).
%
%   [EBME, EFF_DIM] = EBME_PREP(...) also specifies the effective
%   dimension.
%
%   A warning is issued if the problem parameters do not guarantee
%   dominance over least squares.
%
%   See also SBME, EBME_PREP, EBME.

%   Asaf Elron and Guy Leibovitz, 2008.
%   Based on Ben-Haim & Eldar, Blind Minimax Estimation", IEEE TIT, 2007.

%% calculate stuff
if (exist('option', 'var') && option == 1) % were LS and Qinv given?
    % set local variables according to given parameters
    LS = H_or_LS;
    Qinv = Cw_or_Qinv;
    epsilon0 = trace(Qinv);
else
    % calculate LS estimator matrix and Qinv
    [LS, Qinv, epsilon0] = leastsq_prep(H_or_LS, Cw_or_Qinv);
end;

% create the struct and assign output variable
sbme_data = struct('LS', LS, 'epsilon0', epsilon0);

%% check and report on dominance over least squares
epsilon_max = max(eig(Qinv)); % calculate largest eigenvalue of Q_inverse
effective_dimension = epsilon0 / epsilon_max;
if ~(effective_dimension > 4) % condition for lack of dominance
    warning('BME:SBME_PREP:EFF_DIM', ...
        'Effective dimension is %g. This does not guarantee dominance over LS.', ...
        effective_dimension);
end;

end
