function performance = compare_estimators(H, Cw, getX, getY, number_of_samples, b, selection)
%COMPARE_ESTIMATORS Compare estimators in suite.
%   PERFORMANCE = COMPARE_ESTIMATORS(H, Cw, XH, YH, COUNT, B, SEL) produces
%   a structure containing statistics of the performance relative to
%   reference data, for a selection of estimators.
%   The measurement transform H is assumed to be of full rank and the noise
%   covariance Cw is assumed to be positive definite.
%   XH and YH are function handles that accept an index in the range
%   [1,COUNT] and return the original value and the noisy measurement of
%   it, respectively, for that index.
%   B is the discrimination parameter, for estimators for which it
%   applies. If not specified, a discrimination factor of -1 is assumed.
%   SEL is a selection string specifying which of the available
%   estimators to include in the comparison, in which each character
%   represents a different estimator:
%       'l'   least squares, see LEASTSQ
%       's'   spherical blind minimax, see SBME
%       'e'   ellipsoidal blind minimax, see EBME
%   If not specified, a selection string 'lse' is assumed.
%
%   PERFORMANCE = COMPARE_ESTIMATORS(H, Cw, X, Y, COUNT, B, SEL) behaves in
%   the same way as the above, other than X and Y being matrices with COUNT
%   original values and measurements vectors, respectively, as their
%   columns.
%
%   Returned structure includes vectors of the means and of the standard
%   deviations of the squared estimation errors, under the keys
%   'means_of_squared_errors' and 'stddevs_of_squared_errors' respectively.
%   A specific estimator's statistics are positioned within the vectors
%   according to its position in the selection string SEL.
%   Also included in the returned structure is the (theoretical) MSE of the
%   least squares estimator, under the key 'epsilon0'.
%
%   Example:
%       H = eye(5);
%       Cw = diag([1 1 1 1 0.4]);
% 
%       X = [eye(5) eye(5)];
%       Y = H*X + Cw^0.5*randn(size(Cw, 1), size(X, 2));
% 
%       getX = @(i) X(:, i);
%       getY = @(i) Y(:, i);
% 
%       stats = compare_estimators(H, Cw, getX, getY, size(X, 2), -1, 'lse');
%
%   See also LEASTSQ, LEASTSQ_PREP, SBME, SBME_PREP, EBME, EBME_PREP.

%   Asaf Elron and Guy Leibovitz, 2008.
%   Based on Ben-Haim & Eldar, Blind Minimax Estimation", IEEE TIT, 2007.

%% process parameters
if (~isa(getX, 'function_handle')) % was a matrix specified for X, and not a function handle?
    % "convert" for later use
    Xdata = getX;
    getX = @(i) Xdata(:, i);
end;
if (~isa(getY, 'function_handle')) % was a matrix specified for Y, and not a function handle?
    % "convert" for later use
    Ydata = getY;
    getY = @(i) Ydata(:, i);
end;

if (~exist('b', 'var')) % was a discrimination factor not specified?
    b = -1; % assumed -1
end;

if (~exist('selection', 'var')) % was an estimator selection not specified?
    selection = 'lse'; % assume 'lse'
end;

%% prepare estimators for iteration
% find 
do_ls = find(selection == 'l', 1);
do_sbm = find(selection == 's', 1);
do_ebm = find(selection == 'e', 1);

estimators = {};
prep_data = {};
if (~isempty(do_ls))
    LS = leastsq_prep(H, Cw); % generate LS estimator
    estimators{do_ls} = @leastsq;
    prep_data{do_ls} = LS;
end;

if (~isempty(do_sbm))
    sbme_data = sbme_prep(H, Cw); % generate SBME estimator
    estimators{do_sbm} = @sbme;
    prep_data{do_sbm} = sbme_data;
end;

if (~isempty(do_ebm))
    ebme_data = ebme_prep(H, Cw, b); % generate EBME estimator
    estimators{do_ebm} = @ebme;
    prep_data{do_ebm} = ebme_data;
end;

%% iterate through all samples and all estimators, while (naively) collecting data for later statistics
squared_errors = zeros(number_of_samples, size(estimators, 2)); % squared error for every sample and every estimator
for i = 1:number_of_samples % iterate through all samples

    x = getX(i); % get original vector
    y = getY(i); % get (noisy) measurement of above vector
    
    for j = 1:length(estimators) % iterate through all estimators
        
        x_hat = estimators{j}(prep_data{j}, y); % estimate x according to y
        squared_errors(i, j) = norm_squared(x_hat - x); % calculate and store squared error
        
    end;

end;

%% calculate statistics
means_of_squared_errors = mean(squared_errors, 1);
stddevs_of_squared_errors = std(squared_errors, 1);

%% output 'em results
[something, something_else, epsilon0] = leastsq_prep(H, Cw); % get epsilon0
performance = struct('epsilon0', epsilon0, ...
                     'means_of_squared_errors', means_of_squared_errors, ...
                     'stddevs_of_squared_errors', stddevs_of_squared_errors);

end
