function [H, Y, noise_variance] = get_hy(numerator, denomenator, U, snr, preceding_input)
%GET_HY Model matrix and noisy output for given filter and input.
%   [H, Y, NOISE_VAR] = GET_HY(NUM, DEN, U) produces the model matrix H and
%   the noisy output Y, for a given filter specified by the numerator NUM
%   and the denomenator DEN, with the input signal U. Also specified is the
%   variance of the noise used in NOISE_VAR.
%
%   [...] = GET_HY(NUM, DEN, U, SNR) also specifies the signal-to-noise
%   ratio at the filter output. If not specified, an SNR of 1 is assumed.
%
%   [...] = GET_HY(NUM, DEN, U, SNR, PRE_U) also specifies input preceding
%   that specified in U, which can be used to neutralize any effects of the
%   zero initial condition otherwise assumed.
%
%   See also ARX_COMPARISON.

%   Asaf Elron and Guy Leibovitz, 2008.
%   Based on Leibovitz, Elron, Ben-Haim & Eldar, Recursive Blind Minimax
%   Estimation: Improving MSE over Recursive Least Squares", IEEEI 2008.

%% parameter management and setup
if (nargin < 4) % was an snr not specified?
    snr = 1; % assume an snr of 1
end;
if (nargin < 5) % was a preceding input not specified?
    preceding_input = []; % assume no preceding input
end;

N = length(U); % length of input

% ARX(p, q)
p = length(denomenator);
q = length(numerator);

%% calculate noise variance
noise_sys = filt(1, denomenator'); % filter through which the noise passes
input_sys = filt(numerator', denomenator'); % filter through which the input passes

noise_sys_frequnecy_response_mag = @(w) ( bode(noise_sys, w) ); % unnamed function for integration
mean_power_of_output_for_normalized_noise_input = N * quad(noise_sys_frequnecy_response_mag, 0, 2*pi) / (2*pi); % E{|| g ||^2} (5.3)
noise_variance = norm_squared(lsim(input_sys, U)) / (mean_power_of_output_for_normalized_noise_input * snr); % (5.3)
% NOTE: The above is not entirely accurate, since it does not take into
% account the effect of the windowing on the spectral power density of the
% noise when calculating its mean power.

%% calculate noisy output
sys = filt({numerator', 1}, {denomenator', denomenator'}); % system for both input & noise
noise = sqrt(noise_variance)*randn(N+length(preceding_input), 1); % realize the noise input
Y = lsim(sys, [[preceding_input; U] noise]); % the noisy output, including any preceding steps

% rearrange the noisy output
preceding_output = Y(1:length(preceding_input)); % preceding output corresponds in time with preceding input
Y = Y(length(preceding_input)+1:end); % noisy output corresponds in time with input

%% generate model matrix
if (length(preceding_input) < max(p-1, q-1)) % is preceding input not long enough?
    % don't use preceding input in model matrix generation
    H_U = toeplitz(U, [U(1) zeros(1, q-1)]); % specify model matrix for zeros
    if (p-1 > 0) % are there poles?
        H_Y = toeplitz([0; Y(1:end-1)], zeros(1, p-1)); % specify model matrix for poles
    else
        H_Y = [];
    end;
    if (preceding_input ~= [])
        warning('BME:GET_HY:PRE_U', ...
                'Preceding input specified is too short. Please specify at least max(p-1, q-1) values or none at all.');
    end;
else
    % use preceding input in model matrix generation
    H_U = toeplitz(U, [U(1); preceding_input(end-(q-1)+1:end)]); % specify model matrix for zeros
    if (p-1 > 0) % are there poles?
        H_Y = toeplitz([preceding_output(end-(p-1)+1); Y(1:end-1)], preceding_output(end-(p-1)+1:end)); % specify model matrix for zeros
    else
        H_Y = []; % specify model matrix for zeros
    end;
end;

H = [ H_U H_Y ]; % specify model matrix

end
